package de.ozgcloud.bescheid.vorgang;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import lombok.NonNull;

@Service("bescheid_VorgangService")
public class VorgangService {

	static final String VORGANG_BESCHEIDEN = "VORGANG_BESCHEIDEN";

	@Autowired
	private VorgangRemoteService remoteService;
	@Autowired
	@Qualifier("bescheid_OzgCloudCommandService")
	private OzgCloudCommandService commandService;
	@Autowired
	private CommandMapper commandMapper;

	public Vorgang getById(@NonNull VorgangId id) {
		return remoteService.getById(id);
	}

	public void bescheiden(String vorgangId) {
		bescheiden(getById(VorgangId.from(vorgangId)));
	}

	private void bescheiden(Vorgang vorgang) {
		commandService.createAndWaitUntilDone(buildBescheidenCommand(vorgang));
	}

	OzgCloudCommand buildBescheidenCommand(Vorgang vorgang) {
		return OzgCloudCommand.builder()
				.vorgangId(commandMapper.toOzgCloudVorgangId(vorgang.getId().toString()))
				.relationId(commandMapper.mapRelationId(vorgang.getId().toString()))
				.relationVersion(vorgang.getVersion())
				.order(VORGANG_BESCHEIDEN)
				.build();
	}

}
