package de.ozgcloud.bescheid;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.bescheid.attacheditem.AttachedItemService;
import de.ozgcloud.bescheid.common.callcontext.CurrentUserService;
import de.ozgcloud.bescheid.common.user.UserProfile;
import de.ozgcloud.bescheid.common.user.UserProfileService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEventTestFactory;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.common.test.ITCase;

@ITCase
class BescheidEventListenerITCase {

	@Autowired
	private ApplicationEventPublisher publisher;
	@Autowired
	private BescheidFeatureProperties bescheidFeatureProperties;

	@MockBean
	private BescheidService service;
	@MockBean
	private CurrentUserService userService;
	@MockBean
	private UserProfileService userProfileService;
	@MockBean
	private OzgCloudCommandService commandService;
	@MockBean
	private CommandMapper commandMapper;
	@MockBean
	private AttachedItemService attachedItemService;

	@Mock
	private UserProfile userProfile;

	@BeforeEach
	void init() {
		when(userProfileService.getUserProfile()).thenReturn(userProfile);
	}

	@Nested
	class TestCreateBescheid {

		private final Command command = CommandTestFactory.createBuilder().order(BescheidEventListener.CREATE_BESCHEID_ORDER).build();

		@BeforeEach
		void init() {
			bescheidFeatureProperties.setKielHackathonRoute(true);
		}

		@Test
		void shouldCallService() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(command));

			verify(service).createBescheid(any());
		}

		@Test
		void shouldNotReactOnOtherOrder() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(CommandTestFactory.createBuilder().order("OTHER").build()));

			verifyNoInteractions(service);
		}
	}

	@Nested
	class TestDeleteBescheid {

		private final Command command = CommandTestFactory.createBuilder().order(BescheidEventListener.DELETE_BESCHEID_ORDER).build();

		@Test
		void shouldCallService() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(command));

			verify(attachedItemService).deleteBescheidDraft(command);
		}

		@Test
		void shouldNotReactOnOtherOrder() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(CommandTestFactory.createBuilder().order("OTHER").build()));

			verifyNoInteractions(attachedItemService);
		}
	}

	@Nested
	class TestUpdateBescheid {

		private final Command command = CommandTestFactory.createBuilder().order(BescheidEventListener.UPDATE_BESCHEID_ORDER).build();

		@Test
		void shouldCallService() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(command));

			verify(attachedItemService).updateBescheidDraft(any());
		}

		@Test
		void shouldNotReactOnOtherOrder() {
			publisher.publishEvent(CommandCreatedEventTestFactory.withCommand(CommandTestFactory.createBuilder().order("OTHER").build()));

			verifyNoInteractions(attachedItemService);
		}
	}
}
