package de.ozgcloud.bescheid;

import java.io.File;
import java.util.Optional;

import org.springframework.http.MediaType;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.bescheid.common.user.UserProfileTestFactory;
import de.ozgcloud.bescheid.vorgang.ServiceKontoTestFactory;
import de.ozgcloud.bescheid.vorgang.VorgangTestFactory;
import de.ozgcloud.common.binaryfile.FileId;
import de.ozgcloud.common.binaryfile.TempFileUtils;

public class BescheidTestFactory {

	public static final UserId CREATED_BY = UserProfileTestFactory.ID;
	public static final String FILE_NAME = "bescheid.pdf";

	public static final byte[] TEST_BESCHEID = "testbescheid".getBytes();
	public static final File BESCHEID_FILE = TempFileUtils.writeTmpFile(TEST_BESCHEID);
	public static final String BESCHEID_FILE_ID = "bescheid-file-id";

	public static final String CONTENT_TYPE = MediaType.APPLICATION_PDF_VALUE;
	public static final String NACHRICHT_SUBJECT = LoremIpsum.getInstance().getWords(5);
	public static final String NACHRICHT_TEXT = LoremIpsum.getInstance().getWords(5);

	public static Bescheid create() {
		return createBuilder().build();
	}

	public static Bescheid.BescheidBuilder createBuilder() {
		return Bescheid.builder()
				.createdBy(CREATED_BY)
				.vorgangId(VorgangTestFactory.ID)
				.contentType(CONTENT_TYPE)
				.bescheidFileName(FILE_NAME)
				.bescheidFile(BESCHEID_FILE)
				.bescheidFileId(FileId.from(BESCHEID_FILE_ID))
				.nachrichtSubject(Optional.of(NACHRICHT_SUBJECT))
				.nachrichtText(Optional.of(NACHRICHT_TEXT))
				.genehmigt(true)
				.size(TEST_BESCHEID.length)
				.serviceKonto(ServiceKontoTestFactory.create());
	}
}
