/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import java.util.Collections;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.dao.DataAccessResourceFailureException;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import de.ozgcloud.vorgang.common.errorhandling.SearchServiceUnavailableException;
import de.ozgcloud.vorgang.vorgang.FindVorgangRequest;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangHeader;

@Service
@ConditionalOnProperty(prefix = "ozgcloud.elasticsearch", name = "address")
public class SearchService {

	@Autowired
	private SearchVorgangRepository repository;

	@Autowired
	private SearchVorgangCustomRepository searchRepository;

	@Autowired
	private IndexedVorgangMapper mapper;

	public void addVorgang(Vorgang vorgang) {
		repository.save(mapper.fromVorgang(vorgang));
	}

	public void updateVorgang(Vorgang vorgang) {
		repository.save(mapper.fromVorgang(vorgang));
	}

	public boolean existsById(String vorgangId) {
		return repository.existsById(vorgangId);
	}

	public Page<VorgangHeader> find(FindVorgangRequest findVorgangRequest) {
		try {
			return searchRepository.searchBy(findVorgangRequest);
		} catch (DataAccessResourceFailureException e) {
			throw new SearchServiceUnavailableException(e);
		}
	}

	public void deleteVorgang(String vorgangId) {
		repository.deleteById(vorgangId);
	}

	public void setAktenzeichen(String vorgangId, String aktenzeichen) {
		searchRepository.update(vorgangId, Collections.singletonMap(IndexedVorgang.FIELD_AKTENZEICHEN, aktenzeichen));
	}

}