package de.ozgcloud.vorgang.attached_item;

import static org.assertj.core.api.Assertions.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import jakarta.validation.ConstraintViolationException;

@ITCase
class VorgangAttachedItemServiceITCase {

	@Autowired
	private VorgangAttachedItemService service;

	@MockBean
	private VorgangAttachedItemRepository repository;
	@MockBean
	private ApplicationEventPublisher eventPublisher;

	@Nested
	class TestCreate {
		@Test
		void shouldDenyMissingClientName() {
			var item = VorgangAttachedItemTestFactory.createBuilder().client(StringUtils.EMPTY).build();

			assertThatThrownBy(() -> service.create(CommandTestFactory.ID, item))
					.isInstanceOf(ConstraintViolationException.class);
		}

		@Test
		void shouldDenyMissingVorgangId() {
			var item = VorgangAttachedItemTestFactory.createBuilder().vorgangId(StringUtils.EMPTY).build();

			assertThatThrownBy(() -> service.create(CommandTestFactory.ID, item))
					.isInstanceOf(ConstraintViolationException.class);
		}

		@Test
		void shouldDenyMissingItemName() {
			var item = VorgangAttachedItemTestFactory.createBuilder().itemName(StringUtils.EMPTY).build();

			assertThatThrownBy(() -> service.create(CommandTestFactory.ID, item))
					.isInstanceOf(ConstraintViolationException.class);
		}
	}

}
