package de.ozgcloud.vorgang.common.migration;

import static de.ozgcloud.vorgang.common.migration.M002_UpdateClientNameInGridFs.*;
import static org.assertj.core.api.Assertions.*;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Query;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class M002_UpdateClientNameInGridFsITCase {

	private static final Document OLD_CLIENT_NAME_GRID_FS_FILE = RenameTestFactory.createGridFsFile(OLD_CLIENT_NAME);

	private final M002_UpdateClientNameInGridFs migration = new M002_UpdateClientNameInGridFs();

	@Autowired
	private MigrationDbTestUtils dbTestUtils;
	@Autowired
	private MongoOperations db;

	@BeforeEach
	public void init() {
		dbTestUtils.dropGridFsFilesCollection();
	}

	@DisplayName("Do migration")
	@Nested
	class TestDoMigration {

		@BeforeEach
		void init() {
			dbTestUtils.saveGridFsFile(OLD_CLIENT_NAME_GRID_FS_FILE);
		}

		@Test
		void shouldUpdateClientInMetadata() {
			migration.doMigration(db);

			var metadata = (Document) MapUtils.getObject(getGridFsFile(), RenameUtil.METADATA_KEY);
			assertThat(metadata).contains(entry(RenameUtil.CLIENT_KEY, NEW_CLIENT_NAME));
		}

		@Test
		void shouldUpdateClientInFilename() {
			migration.doMigration(db);

			var filename = MapUtils.getString(getGridFsFile(), RenameUtil.FILENAME_KEY);
			var expectedFilename = StringUtils.replace(RenameTestFactory.FILE_NAME_VALUE, OLD_CLIENT_NAME, NEW_CLIENT_NAME);
			assertThat(filename).isEqualTo(expectedFilename);
		}

		@Test
		void shouldKeepBaseData() {
			migration.doMigration(db);

			var gridFsFile = getGridFsFile();
			assertThat(gridFsFile).contains(entry(RenameTestFactory.LENGTH_KEY, RenameTestFactory.LENGTH_VALUE))
					.contains(entry(RenameTestFactory.CHUNK_SIZE_KEY, RenameTestFactory.CHUNK_SIZE_VALUE))
					.contains(entry(RenameTestFactory.UPLOAD_DATE_KEY, RenameTestFactory.UPLOAD_DATE_VALUE));
		}

		@Test
		void shouldKeepMetadata() {
			migration.doMigration(db);

			var metadata = (Document) MapUtils.getObject(getGridFsFile(), RenameUtil.METADATA_KEY);
			assertThat(metadata).contains(entry(RenameTestFactory.VORGANG_ID_KEY, RenameTestFactory.VORGANG_ID_VALUE))
					.contains(entry(RenameTestFactory.FIELD_NAME_KEY, RenameTestFactory.FIELD_NAME_VALUE))
					.contains(entry(RenameTestFactory.CONTENT_TYPE_KEY, RenameTestFactory.CONTENT_TYPE_VALUE))
					.contains(entry(RenameTestFactory.NAME_KEY, RenameTestFactory.NAME_VALUE));
		}

		private Document getGridFsFile() {
			return dbTestUtils.findGridFsFile(new Query()).get(0);
		}
	}
}